@@ -42,6 +42,7 @@
 #define STRICT_MS_STYLE_ZONEIDX 0
 #define MS_SUFFIX "ipv6-literal.net"
 
+#define PTR_2_INT(x) ((x) - ((__typeof__ (x))NULL))
 #define ALIGN(a) (((a+sizeof(void*)-1)/sizeof(void*))*sizeof(void*))
 
 static int check_name(const char *name, char **_addr, char **_fqdn)
@@ -127,6 +128,7 @@ static int get_addr(char *ipv6str, uint8_t *addr, uint32_t *scopeid)
 	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = AF_INET6;
 	hints.ai_socktype = SOCK_STREAM;
+        hints.ai_flags = AI_NUMERICHOST;
 
 	ret = getaddrinfo(ipv6str, NULL, &hints, &res);
 	if (ret != 0) {
@@ -163,7 +165,9 @@ enum nss_status _nss_ipv6literal_gethostbyname4_r(
 	int ret;
 
         l = strlen(name);
-        ms = ALIGN(l+1)+ALIGN(sizeof(struct gaih_addrtuple));
+        ms = ALIGN(l + 1) +
+              ALIGN(sizeof(struct gaih_addrtuple)) +
+              (ALIGN(PTR_2_INT(buffer)) - PTR_2_INT(buffer));
         if (buflen < ms) {
                 *errnop = ENOMEM;
                 *h_errnop = NO_RECOVERY;
@@ -178,8 +182,9 @@ enum nss_status _nss_ipv6literal_gethostbyname4_r(
 	}
 
 	/* set buffer */
-        r_tuple1 = (struct gaih_addrtuple*)buffer;
+        r_tuple1 = (struct gaih_addrtuple*)(ALIGN(PTR_2_INT(buffer)));
         idx = ALIGN(sizeof(struct gaih_addrtuple));
+        r_tuple1->name = (char *)(r_tuple1 + idx);
 
         /* fill in IPv6 addr */
 	ret = get_addr(addr, (uint8_t *)r_tuple1->addr, &r_tuple1->scopeid);
